using UnityEngine;
using System;
using System.IO;
using System.Collections;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;

[Serializable]
public enum FCPadEventType{
    kFCPadEventNone = 0,
    kFCPadEventConnectionRequest,
    kFCPadEventConnectionResponseAccept,
    kFCPadEventConnectionResponseRefuse,
    kFCPadEventApplicationSwitch,
    kFCPadEventKeyDown,
    kFCPadEventKeyUp,
    kFCPadChannelEvent
};

[Serializable]
[StructLayout (LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack=1)]
public struct FCPadChannelInput { //sizeof is 64+(4*3) = 76
    public System.UInt32 min;
    public System.UInt32 max;
    public System.UInt32 value;
    public string lable; //max is 63+1
};

[Serializable]
[StructLayout (LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack=1)]
public struct FCPadChannelData { //sizeof is 4 + 76 + 128 = 208
    public System.UInt32 identifier;
    public FCPadChannelInput input;
    public string name; //max is 127+1
};

[Serializable]
[StructLayout (LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack=1)]
public struct FCPadChannelPacket { //sizeof is 12 + 208 = 220 (220.0 / 4.0 = 55.0, no padding)
    public System.UInt32 size;
    public System.UInt32 type;
    public System.UInt32 dataSize;
    public FCPadChannelData data;
};

[Serializable]
[StructLayout (LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack=1)]
public class CPadChannelEvent : object {

    protected FCPadChannelPacket packet;

    public string masterChannelIdentifier {
        get{
            return String.Copy(packet.data.name);
        }
        set{
            string str = value;
            if(str.Length > 123){
                str = str.Substring(0,123);
            }
            packet.data.name = String.Copy(str);
        }
    }

    public System.UInt32 channelIdentifier {
        get{
            return packet.data.identifier;
        }
        set{
            packet.data.identifier = value;
        }
    }

    public System.UInt32 inputMin {
        get{
            return packet.data.input.min;
        }
        set{
            packet.data.input.min = value;
        }
    }

    public System.UInt32 inputMax {
        get{
            return packet.data.input.max;
        }
        set{
            packet.data.input.max = value;
        }
    }

    public System.UInt32 inputValue {
        get{
            return packet.data.input.value;
        }
        set{
            packet.data.input.value = value;
        }
    }

    public string inputLabel {
        get{
            return String.Copy(packet.data.input.lable);
        }
        set{
			if (value == null)
				return;
            string str = value;
            if(str.Length > 511){
                str = str.Substring(0,511);
            }
            packet.data.input.lable = String.Copy(str);
        }
    }

    public CPadChannelEvent(){
        packet.size = 668;
        packet.type = (System.UInt32)FCPadEventType.kFCPadChannelEvent;
        packet.data.name = "";
        packet.data.input.lable = "";
        packet.dataSize = 656;
    }

    public byte[] ToBytes()
    {
        byte[] pkt = new byte[packet.size];
        int offset = 0;

        //size
        byte[] bytes = BitConverter.GetBytes(packet.size);
        if (!BitConverter.IsLittleEndian)
            Array.Reverse(bytes);
        Buffer.BlockCopy(bytes, 0, pkt, offset, 4); offset += 4;

        //type
        bytes = BitConverter.GetBytes(packet.type);
        if (!BitConverter.IsLittleEndian)
            Array.Reverse(bytes);
        Buffer.BlockCopy(bytes, 0, pkt, offset, 4); offset += 4;

        bytes = BitConverter.GetBytes(packet.dataSize);
        if (!BitConverter.IsLittleEndian)
            Array.Reverse(bytes);
        Buffer.BlockCopy(bytes, 0, pkt, offset, 4); offset += 4;

        //identifier
        bytes = BitConverter.GetBytes(packet.data.identifier);
        if (!BitConverter.IsLittleEndian)
            Array.Reverse(bytes);
        Buffer.BlockCopy(bytes, 0, pkt, offset, 4); offset += 4;

        //inputMin
        bytes = BitConverter.GetBytes(packet.data.input.min);
        if (!BitConverter.IsLittleEndian)
            Array.Reverse(bytes);
        Buffer.BlockCopy(bytes, 0, pkt, offset, 4); offset += 4;

        //inputMax
        bytes = BitConverter.GetBytes(packet.data.input.max);
        if (!BitConverter.IsLittleEndian)
            Array.Reverse(bytes);
        Buffer.BlockCopy(bytes, 0, pkt, offset, 4); offset += 4;

        //inputValue
        bytes = BitConverter.GetBytes(packet.data.input.value);
        if (!BitConverter.IsLittleEndian)
            Array.Reverse(bytes);
        Buffer.BlockCopy(bytes, 0, pkt, offset, 4); offset += 4;

        //inputLable
        bytes = System.Text.UTF8Encoding.UTF8.GetBytes( packet.data.input.lable );
        int sz = bytes.Length;
        Buffer.BlockCopy(bytes, 0, pkt, offset, sz); offset += 512;//fixed size string

        //Master Identifier
        bytes = System.Text.UTF8Encoding.UTF8.GetBytes( packet.data.name );
        sz = bytes.Length;
        Buffer.BlockCopy(bytes, 0, pkt, offset, sz); offset += 128;//fixed size string

        //Debug.Log(BitConverter.ToString(pkt));

        return pkt;
    }

    public override string ToString() {
        return 
            "\"" + masterChannelIdentifier + "\"\n"+
            channelIdentifier + "\n"+
            inputMin +"\n"+
            inputMin+"\n"+
            inputMax+"\n"+
            inputValue+"\n"+
            "\""+ inputLabel + "\"\n"
        ;
    }
}
