using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using UnityEngine;
using System.Net;
using System.Net.Sockets;
namespace CCPConnector
{
	public enum CCPMessageType{String,Value,RangeValue,Other };
	public enum InfoType{Vessel,Resources,Orbit,Body,Other };
	public class KCCPPacket
	{
	
		public bool active;
		public bool debug;
		public string description;
		public string valString;
		public int valInt;
		public int min;
		public int max;
		public GameScenes sceneType;
		public CCPMessageType type = CCPMessageType.String;
		//scene type in future ? so to send certain messages in certain scenes only ?
		// switch layout in CCP via code ?
		public bool cPrivate;
		CPadChannelEvent channelEvent;
		public float sendRate = 0.5f;
		public float nextSendTime;
		public bool disableSendRateCheck;
		private byte[] bytes;
		public KCCPPacket() {
			active = true;
			channelEvent = new CPadChannelEvent();
		}
		public KCCPPacket(string mId,int chId,GameScenes s) : this(mId,chId) {
			sceneType = s;
			//KCCPPacket(mId,chId);
		}
		public KCCPPacket (string mId,int chId ) {
			active = true;
			channelEvent = new CPadChannelEvent();
			channelEvent.masterChannelIdentifier = mId; 
			channelEvent.channelIdentifier = (uint) Convert.ToSByte(chId);
		}	
		public KCCPPacket (int chId ) {
			active = true;
			channelEvent = new CPadChannelEvent();	
			channelEvent.channelIdentifier = (uint) Convert.ToSByte(chId);
			//REMEMEBR !!! SET THE masterChannelIdentifier else......
		}
		public KCCPPacket (int chId,CCPMessageType tp,GameScenes s) : this(chId,tp){
			sceneType = s;
		//	KCCPPacket(chId,tp);	
		}
		public KCCPPacket (int chId,CCPMessageType tp) : this(chId) {
			type = tp;
			//REMEMEBR !!! SET THE masterChannelIdentifier else......
		}
		public void SetMasterCHId(string id) {
			channelEvent.masterChannelIdentifier = id;	
		}	
		private bool sendCheck;
		public bool Communicate(Socket sk) {
			if (Time.time < nextSendTime && disableSendRateCheck == false)
				return false;
			sendCheck = true;
			switch(type) {
				case CCPMessageType.String :
					if (channelEvent.inputLabel == valString)
						return false;
					channelEvent.inputLabel = valString;
				break;
				case CCPMessageType.Value :
					if (channelEvent.inputValue == (uint) valInt)
						return false;
					channelEvent.inputValue =	(uint) valInt;
				break;
				case CCPMessageType.RangeValue : 
					if (channelEvent.inputValue == (uint) valInt && channelEvent.inputMin ==  (uint)min && channelEvent.inputMax == (uint)max && channelEvent.inputLabel == valString)
						return false;
					channelEvent.inputValue = (uint) valInt;
					channelEvent.inputMin = (uint)min;
					channelEvent.inputMax = (uint)max;
					channelEvent.inputLabel = valString;
					if(max <= 0)
						return false;
				break;
			}
			
			bytes = channelEvent.ToBytes();
			if (bytes == null) {
				Debug.Log("KCCPPackt  Communicating on ch : "  + channelEvent.channelIdentifier + " val = "  + channelEvent.inputLabel + " bytes == null....");
				return false;
			}	
			if (debug) {
				switch (type) {
					case CCPMessageType.String : 
						Debug.Log("KCCPPackt  Communicating on ch : "  + channelEvent.channelIdentifier + " val = "  + channelEvent.inputLabel);
					break;
					case CCPMessageType.Value : 
						Debug.Log("KCCPPackt  Communicating on ch : "  + channelEvent.channelIdentifier + " val = "  + channelEvent.inputValue);
					break;
				}	
			}
			
			sk.Send(bytes,bytes.Length, 0 );	
			nextSendTime = Time.time + sendRate;
			return true;
		}
	}	
	
}

